@@ -159,6 +159,9 @@ static struct Settings {
         XATTR_READ_WRITE
     } xattr_policy;
 
+    int delete_deny;
+    int rename_deny;
+
     int mirrored_users_only;
     uid_t *mirrored_users;
     int num_mirrored_users;
@@ -184,7 +187,7 @@ static struct Settings {
     int enable_lock_forwarding;
 
     int enable_ioctl;
-
+    
     uid_t uid_offset;
     gid_t gid_offset;
 
@@ -476,6 +479,9 @@ static int delete_file(const char *path, int (*target_delete_func)(const char *)
     char *unlink_first = NULL;
     int (*main_delete_func)(const char*) = target_delete_func;
 
+     if (settings.delete_deny)
+        return -EPERM;
+
     real_path = process_path(path, false);
     if (real_path == NULL)
         return -errno;
@@ -678,6 +684,10 @@ static int bindfs_readdir(const char *path, void *buf, fuse_fill_dir_t filler,
     if (pc_ret < 0) {
         DPRINTF("pathconf failed: %s (%d)", strerror(errno), errno);
         pc_ret = NAME_MAX;
+    } else if (pc_ret == 0) {
+        // Workaround for some source filesystems erroneously returning 0
+        // (see issue #54).
+        pc_ret = NAME_MAX;
     }
 
     struct dirent *de_buf =
@@ -813,6 +823,9 @@ static int bindfs_rename(const char *from, const char *to)
     int res;
     char *real_from, *real_to;
 
+    if (settings.rename_deny)
+        return -EPERM;
+
     real_from = process_path(from, false);
     if (real_from == NULL)
         return -errno;
@@ -1454,6 +1467,10 @@ static void print_usage(const char *progname)
            "  --xattr-ro                Read-only xattr operations.\n"
            "  --xattr-rw                Read-write xattr operations (the default).\n"
            "\n"
+           "Other file operations:\n"
+           "  --delete-deny             Disallow deleting files.\n"
+           "  --rename-deny             Disallow renaming files (within the mount).\n"
+           "\n"
            "Rate limits:\n"
            "  --read-rate=...           Limit to bytes/sec that can be read.\n"
            "  --write-rate=...          Limit to bytes/sec that can be written.\n"
@@ -1504,6 +1521,8 @@ enum OptionKey {
     OPTKEY_XATTR_NONE,
     OPTKEY_XATTR_READ_ONLY,
     OPTKEY_XATTR_READ_WRITE,
+    OPTKEY_DELETE_DENY,
+    OPTKEY_RENAME_DENY,
     OPTKEY_REALISTIC_PERMISSIONS,
     OPTKEY_CTIME_FROM_MTIME,
     OPTKEY_ENABLE_LOCK_FORWARDING,
@@ -1583,6 +1602,13 @@ static int process_option(void *data, const char *arg, int key,
         settings.xattr_policy = XATTR_READ_WRITE;
         return 0;
 
+    case OPTKEY_DELETE_DENY:
+        settings.delete_deny = 1;
+        return 0;
+    case OPTKEY_RENAME_DENY:
+        settings.rename_deny= 1;
+        return 0;
+
     case OPTKEY_REALISTIC_PERMISSIONS:
         settings.realistic_permissions = 1;
         return 0;
@@ -1935,6 +1961,9 @@ int main(int argc, char *argv[])
         OPT2("--xattr-ro", "xattr-ro", OPTKEY_XATTR_READ_ONLY),
         OPT2("--xattr-rw", "xattr-rw", OPTKEY_XATTR_READ_WRITE),
 
+        OPT2("--delete-deny", "delete-deny", OPTKEY_DELETE_DENY),
+        OPT2("--rename-deny", "rename-deny", OPTKEY_RENAME_DENY),
+
         OPT2("--hide-hard-links", "hide-hard-links", OPTKEY_HIDE_HARD_LINKS),
         OPT2("--resolve-symlinks", "resolve-symlinks", OPTKEY_RESOLVE_SYMLINKS),
         OPT_OFFSET2("--resolved-symlink-deletion=%s", "resolved-symlink-deletion=%s", resolved_symlink_deletion, -1),
@@ -1946,9 +1975,12 @@ int main(int argc, char *argv[])
         OPT2("--disable-lock-forwarding", "disable-lock-forwarding", OPTKEY_DISABLE_LOCK_FORWARDING),
         OPT2("--enable-ioctl", "enable-ioctl", OPTKEY_ENABLE_IOCTL),
         OPT_OFFSET2("--multithreaded", "multithreaded", multithreaded, -1),
-
         OPT_OFFSET2("--uid-offset=%s", "uid-offset=%s", uid_offset, 0),
         OPT_OFFSET2("--gid-offset=%s", "gid-offset=%s", gid_offset, 0),
+        
+        
+        
+        
         FUSE_OPT_END
     };
 
@@ -1979,6 +2011,8 @@ int main(int argc, char *argv[])
     settings.chmod_allow_x = 0;
     settings.chmod_permchain = permchain_create();
     settings.xattr_policy = XATTR_READ_WRITE;
+    settings.delete_deny = 0;
+    settings.rename_deny = 0;
     settings.mirrored_users_only = 0;
     settings.mirrored_users = NULL;
     settings.num_mirrored_users = 0;
